﻿// FIVe3DMultilineWordWrapAS3FP10 - This is a quick demo of the modified version of multiline wordwrap to support the current version of FIVe3D AS3 and FP10
// Currently supports both FIVe3D_package_AS3_v1.1.3 and FIVe3D_package_AS3_F10_v1.0.2
// This demo uses TweenMax Version: 10.12
// 
// released under MIT License, http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.1.0
//@website 		http://blog.almostlogical.com

//Copyright (c) 2010 Devin Reimer
/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package com 
{
	import fl.controls.Button;
	import fl.controls.CheckBox;
	import flash.display.Shape;
	import flash.display.StageScaleMode;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	
	import fl.controls.Slider;
	import fl.events.SliderEvent;
	import fl.controls.RadioButton;
	import fl.controls.RadioButtonGroup;
	
	import net.badimon.five3D.display.Scene3D;
	import net.badimon.five3D.display.Shape3D;
	import net.badimon.five3D.display.Sprite3D;
	import net.badimon.five3D.typography.HelveticaBold;
	import net.badimon.five3D.typography.HelveticaMedium;
	import net.badimon.five3D.utils.DrawingUtils;
	
	import gs.TweenMax;
	import gs.easing.*
	
	import almostlogical.five3d.display.DynamicText3DMultiline;

	public class Main extends Sprite 
	{
		private static const PAPER_SCALE:int = 30;
		private static const PAPER_WIDTH:int = 8.5 * PAPER_SCALE;
		private static const PAPER_HEIGHT:int = 11 * PAPER_SCALE;
		private static const TITLE_TEXT:String = "Dynamic Word Wrap in FIVe3D";
		private static const BODY_TEXT:String = "This is my upgraded FIVe3D dynamic word wrapping demo. This example demonstrates how easy this new class is to use. I hope you have enjoyed this demo.";
		private static const FOOTER_TEXT:String = "blog.almostlogical.com";
		private static const HEADER_STAR_TEXT:String = "NEW AS3 & FP10";
		private static const MAX_WIDTH:int = 400;
		private static const MIN_WIDTH:int = 75;
		private static const BACKGROUND_COLOUR:uint = 0x3A84EF;
		private var screenWidth:int; //the width of the screen
		private var screenHeight:int; //the height of the screen
		private var baseScene:Sprite3D; //base bottom level container over everything, so it is be rotated around the center
		private var bodyText:DynamicText3DMultiline; //main body of text used to demonstrate word wrap	
		private var bodyTextBackground:Sprite3D;
		private var currentBodyTextWidth:int = 200;
		private var helveticaBold:HelveticaBold = new HelveticaBold();
		private var helveticaMedium:HelveticaMedium = new HelveticaMedium();
		private var isTextVisible:Boolean = true; //if text has been tweened away or not
		private var tweenTextBtn:Button = new Button();
		
		public function Main() 
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;	
			screenWidth = stage.stageWidth;
			screenHeight = stage.stageHeight;
			
			graphics.beginFill(BACKGROUND_COLOUR);
			graphics.drawRect(0, 0, screenWidth, screenHeight);
			graphics.endFill();
			
			setup();
			createSlider();
			createAlignToggle();
			createScatterButton();
			
			baseScene.rotationY -= 20;
			tweenBaseScene();
	
			//addChild(new FPS());
		}
		
		private function setup():void
		{
			var scene:Scene3D = new Scene3D();
			var enviro:Sprite3D = new Sprite3D(); //used to contain all items, it has an offset which by rotating baseScene which contain it, it will rotate on it center axis
			var paper:Sprite3D;
			var mainTitle:Sprite3D;
			var body:Sprite3D;
			var footer:DynamicText3DMultiline;
			var headerVersionGraphic:Sprite3D;
			
			baseScene = new Sprite3D();
			
			//offset baseScene and enviro as top left corner is 0,0 and rotating baseScene rotates everything on it middle axis
			baseScene.x = screenWidth / 2;
			baseScene.y = screenHeight / 2;
			enviro.x = -screenWidth / 2;
			enviro.y = -screenHeight / 2;
			baseScene.addChild(enviro);
			scene.addChild(baseScene);
		
			paper = createPaper();
			paper.x = (screenWidth - PAPER_WIDTH) / 2;
			paper.y = ((screenHeight - PAPER_HEIGHT) / 2)-20;
			enviro.addChild(paper);
			
			mainTitle = createTitle();
			mainTitle.x = 10;
			mainTitle.y = 10;
			paper.addChild(mainTitle);
			
			body = createBody();
			body.y = 100;
			paper.addChild(body);
			
			footer = createFooter();
			footer.x = PAPER_WIDTH - footer.textWidth - 20;
			footer.y = PAPER_HEIGHT - footer.textHeight - 15;
			paper.addChild(footer);
			
			headerVersionGraphic = createHeaderVersionGraphic();
			headerVersionGraphic.x = PAPER_WIDTH - 6;
			headerVersionGraphic.y = 5;
			paper.addChild(headerVersionGraphic);
			
			
			addChild(scene);	
		}
		
		private function createPaper():Sprite3D
		{
			var paper:Sprite3D = new Sprite3D();
			paper.graphics3D.beginFill(0xFFFFFF);
			paper.graphics3D.drawRoundRect(0,0,PAPER_WIDTH,PAPER_HEIGHT, 50, 50);
			paper.graphics3D.endFill();
			return paper;
		}
		
		private function createTitle():Sprite3D
		{
			var sign:Sprite3D = new Sprite3D();
			var firstName:DynamicText3DMultiline;
			var lastName:DynamicText3DMultiline 
			var textY:int = 10;
			
			// We create a new DynamicText3D named "world", modify its properties, place it and add it to the "sign" display list.
			firstName = new DynamicText3DMultiline(helveticaBold); //even though wordwrapping is not need
			firstName.size = 20;
			firstName.color = 0xFFFFFF;
			firstName.text = "Devin";
			firstName.x = 3;
			firstName.y = textY;
			
			lastName = new DynamicText3DMultiline(helveticaBold); 
			lastName.size = 20;
			lastName.color = 0x000000;
			lastName.text = "Reimer";
			lastName.x = firstName.x + 60;
			lastName.y = textY;	
			
			sign.graphics3D.beginFill(0x2503AF);
			sign.graphics3D.drawRoundRect(0,0, 60, 60, 25, 25);
			sign.graphics3D.endFill();			
			
			sign.addChild(firstName);
			sign.addChild(lastName);
			
			return sign;
		}
		
		private function createBody():Sprite3D
		{
			var body:Sprite3D = new Sprite3D();
			var bodyTitle:DynamicText3DMultiline;
			var margin:int = 2;
		
			bodyTitle= new DynamicText3DMultiline(helveticaBold);
			bodyTitle.size = 15;
			bodyTitle.color = 0x000000;
			bodyTitle.text = TITLE_TEXT;
			bodyTitle.x = (PAPER_WIDTH - bodyTitle.textWidth) / 2;
			
			bodyText = new DynamicText3DMultiline(helveticaMedium);
			bodyText.width = currentBodyTextWidth;
			bodyText.size = 14;
			bodyText.letterSpacing = 8;
			bodyText.lineSpacing = 5; //new line space functionality
			bodyText.align = DynamicText3DMultiline.LEFT; //in this case this is not need, just for example
			bodyText.color = 0x000000;
			bodyText.wordWrap = true;
			bodyText.text = BODY_TEXT;
			bodyText.y = margin;
			bodyText.x = margin;
			
			bodyTextBackground = new Sprite3D(); 
			drawBodyTextBackground();
			bodyTextBackground.y = 40;
			
			body.addChild(bodyTitle);
			bodyTextBackground.addChild(bodyText);
			body.addChild(bodyTextBackground);
			
			return body;
		}
		
		private function createFooter():DynamicText3DMultiline
		{
			var almostLogicalText:DynamicText3DMultiline = new DynamicText3DMultiline(helveticaMedium);
			almostLogicalText.size = 10;
			almostLogicalText.text = FOOTER_TEXT;
			
			return almostLogicalText;
		}
		
		private function createHeaderVersionGraphic():Sprite3D
		{
			var headerGraphic:Sprite3D = new Sprite3D();
			var star:Shape3D = new Shape3D();
			var versionText:DynamicText3DMultiline = new DynamicText3DMultiline(helveticaBold);
			var starRadius = 28;
			var starInnerRadius = 22;
			
			versionText.width = 40;
			versionText.size = 10;
			versionText.letterSpacing = 2;
			versionText.lineSpacing = -12;
			versionText.align = DynamicText3DMultiline.CENTER; //in this case this is not need, just for example
			versionText.color = 0xFFFFFF;
			versionText.wordWrap = true;
			versionText.text = HEADER_STAR_TEXT;
			versionText.x = -48;
			versionText.y = 12;
			
			DrawingUtils.starPlace(star.graphics3D, -starRadius, starRadius, 14, starRadius, starInnerRadius, 0, 0x3A84EF);
			headerGraphic.addChild(star);
			headerGraphic.addChild(versionText);
			
			return headerGraphic;
		}
				
		private function tweenBaseScene():void
		{
			var newRotationY:int;
			var swingAngle:Number = 50;
			if (baseScene.rotationY < 0)
			{
				newRotationY = baseScene.rotationY + swingAngle;
			}
			else
			{
				newRotationY = baseScene.rotationY - swingAngle;
			}
			TweenMax.to(baseScene, 6, { rotationY:newRotationY, ease:Back.easeInOut, onComplete:tweenBaseScene } );
		}
		
		private function drawBodyTextBackground():void
		{
			var margin:int = 2;
			var newBackgroundWidth:int = currentBodyTextWidth + (margin * 2);
			bodyTextBackground.graphics3D.clear();
			bodyTextBackground.graphics3D.beginFill(0x81B0F5);
			bodyTextBackground.graphics3D.drawRect(0, 0,newBackgroundWidth, bodyText.height + (margin*2));
			bodyTextBackground.graphics3D.endFill();
			bodyTextBackground.x = (PAPER_WIDTH - newBackgroundWidth) / 2;
		}
		
		private function createAlignToggle():void
		{
			var groupContainer:Sprite = new Sprite();
			var alignRadioGroup:RadioButtonGroup = new RadioButtonGroup("alignGroup");
			var leftAlignRadio:RadioButton = new RadioButton();
			var rightAlignRadio:RadioButton = new RadioButton();
			var centerAlignRadio:RadioButton = new RadioButton();
			
			leftAlignRadio.group = alignRadioGroup;
			rightAlignRadio.group = alignRadioGroup;
			centerAlignRadio.group = alignRadioGroup;
			
			leftAlignRadio.label = "Left Align";
			rightAlignRadio.label = "Right Align";
			centerAlignRadio.label = "Center Align";
	
			leftAlignRadio.value  = DynamicText3DMultiline.LEFT;
			rightAlignRadio.value  = DynamicText3DMultiline.RIGHT;
			centerAlignRadio.value  = DynamicText3DMultiline.CENTER;
			
			leftAlignRadio.selected = true;
			
			alignRadioGroup.addEventListener(Event.CHANGE, updateAlign, false, 0, true);
			
			rightAlignRadio.y = 30;
			centerAlignRadio.y = 60;
			
			groupContainer.addChild(leftAlignRadio);
			groupContainer.addChild(rightAlignRadio);
			groupContainer.addChild(centerAlignRadio);
			
			groupContainer.x = 10;
			groupContainer.y = 10;
			addChild(groupContainer);
		}
		
		private function createSlider():void
		{
			var slider:Slider = new Slider();
			var descriptionLabel:TextField = new TextField();
			
			slider.addEventListener(SliderEvent.CHANGE, changeTextBoxWidth);
			slider.minimum = MIN_WIDTH;
			slider.maximum = MAX_WIDTH;
			slider.value = currentBodyTextWidth;
			slider.width = 200;
			slider.x = (screenWidth - slider.width) / 2;
			slider.y = screenHeight - slider.height - 20;
	
			descriptionLabel.text = "Move the slider to adjust the width of the dynamic text";
			descriptionLabel.autoSize = TextFieldAutoSize.LEFT;
			descriptionLabel.x = slider.x + (slider.width - descriptionLabel.width) / 2;
			descriptionLabel.y = slider.y - descriptionLabel.height - 6;
	
			addChild(slider);
			addChild(descriptionLabel);
		}
		
		private function changeTextBoxWidth(evt:SliderEvent):void
		{
			resetTextIfNecessary(); //due to possiblity of text been 'blown away'
			currentBodyTextWidth = evt.value;
			bodyText.width = currentBodyTextWidth;
			drawBodyTextBackground();
		}
		
		private function updateAlign(evt:Event):void
		{
			resetTextIfNecessary(); //due to possiblity of text been 'blown away'
			bodyText.align = RadioButtonGroup(evt.target).selectedData as String;
		}
			
		private function tweenLetter(shape:Shape3D):void
		{
			TweenMax.to(shape, 4, 
			{ 	x:(currentBodyTextWidth/2) + (Math.random() * 500) - 250,
				y:(bodyText.textHeight/2) + (Math.random() * 500) - 250,
				z:-((Math.random() * 240)+30),
				rotationX:(Math.random() * 360) - 180,
				rotationY:(Math.random() * 360) - 180,
				rotationZ:(Math.random() * 360) - 180
			} );
		}
		
		private function resetShapePosition(shape:Shape):void
		{
			shape.rotationX = 0;
			shape.rotationY = 0;
			shape.rotationZ = 0;
			shape.x = 0;
			shape.y = 0;
			shape.z = 0;
		}
		
		private function createScatterButton():void
		{
			var groupContainer:Sprite = new Sprite();
			tweenTextBtn = new Button();

			setTweenTextButtonText();
			tweenTextBtn.addEventListener(MouseEvent.CLICK, tweenAllText, false, 0, true);

			tweenTextBtn.y = 110;
			tweenTextBtn.x = 15;
			addChild(tweenTextBtn);
		}
		
		private function tweenAllText(evt:Event):void
		{			
			if (isTextVisible) //blow away text
			{
				for (var i:int = 0; i < bodyText.numChildren; i++)
				{
					tweenLetter(bodyText.getChildAt(i) as Shape3D);
				}
			}
			else
			{
				resetText(true);
			}
			
			isTextVisible = !isTextVisible;
			setTweenTextButtonText();
		}
		
		private function resetTextIfNecessary():void
		{
			if (!isTextVisible)
			{
				resetText(false);
				isTextVisible = true;
				setTweenTextButtonText();
			}
		}
		private function resetText(reposition:Boolean):void
		{
			var letter:Shape3D;
			for (var i:int = 0; i < bodyText.numChildren; i++)
			{
				letter = bodyText.getChildAt(i) as Shape3D;
				TweenMax.killTweensOf(letter); 
				resetShapePosition(letter);
			}
			if (reposition)
			{
				bodyText.resetLetterPositions();
			}
		}
		
		private function setTweenTextButtonText():void
		{
			if (isTextVisible)
				tweenTextBtn.label = "Blow Away Text";
			else
				tweenTextBtn.label = "Reset Text";
		}
	}	
}